import 'package:xdja_bridge/gprs_server/protocol/base_protocol/base_request.dart';
import 'package:xml/xml.dart';

/// 录入请求协议
class EntryRequest with BaseRequest {
  /// 请求类别,不能为空
  String reqType;

  /// 会话ID，不能为空
  String sessionID;

  /// 协议版本
  String version;

  /// 业务类型，一般为1
  String transactionType;

  /// 操作表及数据list
  List<Table> tableList;

  /// 添加要操作的表信息，及数据
  /// @param tableName     要操作的表名称
  /// @param isMain        是否是主表，同时添加多个表操作，只允许其中一个表示主表
  /// @param tableDataList 插入，删除，更新操作的数据list
  void addTable(String tableName, bool isMain, List<TableData> tableDataList) {
    if (tableList == null) {
      tableList = List();
    }
    Table table = Table();
    table.tableName = tableName;
    table.isMain = isMain;
    table.tableDataList = tableDataList;
    tableList.add(table);
  }

  String buildXml() {
    var builder = XmlBuilder();
    builder.declaration(encoding: 'UTF-8');
    builder.element('Root', nest: () {
      if (appKey != null && appKey.isNotEmpty) {
        builder.element('AppKey', nest: appKey);
      }
      if (reqDigest != null && reqDigest.isNotEmpty) {
        builder.element('ReqDigest', nest: reqDigest);
      }
      builder.element('ReqType', nest: reqType);
      builder.element('SessionID', nest: sessionID);
      builder.element('TransationType', nest: transactionType);
      builder.element('Tables', nest: () {
        for (Table table in tableList) {
          builder.element('Table', nest: () {
            builder.element('Name', nest: table.tableName);
            builder.element('IsMain', nest: table.isMain);
            for (TableData tableData in table.tableDataList) {
              builder.element('Row', nest: () {
                builder.element('Operate', nest: tableData.operate);
                builder.element('Data', nest: () {
                  for (EntryData entryData in tableData.entryDataList) {
                    if (entryData.isSpecial) {
                      builder.element(entryData.key,
                          nest: {builder.cdata(entryData.value)});
                    } else {
                      builder.element(entryData.key, nest: entryData.value);
                    }
                  }
                });
                builder.element('Condition',
                    nest: {builder.cdata(tableData.condition)});
              });
            }
          });
        }
      });
    });
    return builder.buildDocument().toXmlString();
  }
}

/// 要操作的表信息
class Table {
  /// 表名称
  String tableName;

  ///  是否是主表
  bool isMain;

  ///  存放要插入，删除，更新操作的数据list
  List<TableData> tableDataList;
}

/// 对用数据库表中的一条数据
/// 更新操作dataList数据不能为空，condition可为空
/// 删除、更新操作，dataList数据可以是空的，但是condition条件不能为空
class TableData {
  ///  操作类型
  String operate;

  /// 操作条件，例如：c_xm='张三' and c_xb='1'
  String condition;

  /// 存放操作数据，一个Data代表数据库中一条数据某一列的中的值
  List<EntryData> entryDataList;
  TableData(String operate) {
    this.operate = operate;
  }

  /// 将数据为vaule，添加到column名为key的一条数据中
  /// @param key   对应数据库中column名称
  ///  @param value column中存放的数据
  /// @param isSpecial 包含特殊字符的字段，需要使用CDATA包含的数据
  void addEntryData(String key, String value, {bool isSpecial}) {
    if (entryDataList == null) {
      entryDataList = List();
    }
    entryDataList.add(EntryData(key, value, isSpecial: isSpecial));
  }

  /// 添加一个dataList列表，不会清除掉通过{@link #addEntryData(String, String)}
  /// 方法添加的数据

  void addEntryDataList(List<EntryData> entryDataList) {
    if (this.entryDataList == null) {
      this.entryDataList = List();
    }
    this.entryDataList.addAll(entryDataList);
  }
}

/// 代表一个字段，若{@link #isSpecial 为true，该字段需要使用CADTA包含}
class EntryData {
  ///  字段key
  String key;

  /// 字段值
  String value;

  /// 是否包含特殊字符
  bool isSpecial;
  EntryData(String key, String value, {bool isSpecial: false}) {
    this.key = key;
    this.value = value;
    this.isSpecial = isSpecial;
  }
}
