import 'package:xdja_bridge/gprs_server/protocol/base_protocol/base_request.dart';
import 'package:xml/xml.dart';

/// GPRS Server 登录协议
class QueryRequest with BaseRequest {
  /// 请求类别
  String reqType;

  /// 会话ID
  String sessionID;

  /// 起始记录数,默认为1
  String beginNo = '1';

  /// 请求的表名（可选字段）
  String reqTable;

  /// 请求的字段（可选字段）
  String fieldList;

  /// 数据源，默认一般为1（可选字段，可填*号）
  String dataSource = '*';

  /// 查询字段，每一个查询正常情况应至少包含一个查询字段
  /// 若未添加任何查询条件，未避免后台报错，默认条件为：1=1(查询所有数据)
  List<Condition> conditionList;

  /// 查询条件condition(拼接字符串,与conditionList二选一)
  String condition;

  /// 是否是关联查询(若是关联查询，则会在查询条件外包一层CDATA)
  bool isRelation;

  /// 排序节点
  Order order;

  QueryRequest addCondition(String key, String value,
      {String type, String relation, String splitJoint}) {
    if (conditionList == null) {
      conditionList = List();
    }
    Condition condition = Condition();
    condition.key = key;
    condition.value = value;
    condition.type = type;
    condition.relation = relation;
    condition.splitJoint = splitJoint;
    conditionList.add(condition);
    return this;
  }

  String buildXml() {
    var builder = XmlBuilder();
    builder.declaration(encoding: 'UTF-8');
    builder.element('Root', nest: () {
      if (appKey != null && appKey.isNotEmpty) {
        builder.element('AppKey', nest: appKey);
      }
      if (reqDigest != null && reqDigest.isNotEmpty) {
        builder.element('ReqDigest', nest: reqDigest);
      }
      if (policeNo != null && policeNo.isNotEmpty) {
        builder.element('PoliceNo', nest: policeNo);
      }
      builder.element('ReqType', nest: reqType);
      builder.element('SessionID', nest: sessionID);
      builder.element('BeginNo', nest: beginNo);
      if (reqTable != null && reqTable.isNotEmpty) {
        builder.element('ReqTable', nest: reqTable);
      }
      if (fieldList != null && reqTable.isNotEmpty) {
        builder.element('FieldList', nest: fieldList);
      }
      if (dataSource != null && dataSource.isNotEmpty) {
        builder.element('DataSource', nest: dataSource);
      }
      builder.element('Condition', nest: () {
        if (condition != null && condition.isNotEmpty) {
          if (isRelation) {
            builder.cdata(condition);
          } else {
            builder.text(condition);
          }
        } else if (conditionList != null && conditionList.isNotEmpty) {
          conditionList.forEach((element) {
            builder.element(element.key, nest: () {
              if (element.type != null && element.type.isNotEmpty) {
                builder.attribute('Type', element.type);
              }
              if (element.relation != null && element.relation.isNotEmpty) {
                builder.attribute('Relation', element.relation);
              }
              if (element.splitJoint != null && element.splitJoint.isNotEmpty) {
                builder.attribute('SplitJoint', element.splitJoint);
              }
              builder.text(element.value);
            });
          });
        } else {
          builder.text('1=1');
        }
      });
      if (order != null) {
        builder.element('Order', nest: () {
          if (order.asc != null && order.asc.isNotEmpty) {
            builder.element('asc', nest: order.asc);
          }
          if (order.desc != null && order.desc.isNotEmpty) {
            builder.element('desc', nest: order.desc);
          }
          if (order.mix != null && order.mix.isNotEmpty) {
            builder.element('mix', nest: order.mix);
          }
        });
      }
    });
    var queryRequestXml = builder.buildDocument();
    return queryRequestXml.toXmlString();
  }
}

/// 查询条件
///
/// 每一个查询正常情况应至少包含一个查询字段
/// 若未添加任何查询条件，未避免后台报错，默认条件为：1=1(查询所有数据)
/// 如果要避免查询所有数据请在查询前做校验
/// 查询举例：查询姓名叫"张三"，性别是男的，年龄在18-20之间的人
/// <Condition>
///   <c_xm>张三</c_xm>
///   <c_xb>1</c_xb>
///   <n_age Type='number' Relation='GE'>18</n_age>
///   <n_age Type='number' Relation='LE'>20</n_age>
/// </Condition>
class Condition {
  /// 查询字段key,不可空
  String key;

  /// 查询字段value 不可空
  String value;

  /// 查询字段类型，可空
  /// 后台默认按 [TypeNumberExpression] 中char或1类型
  String type;

  /// 查询字段逻辑关系，可空
  /// 后台默认按{@link com.xdja.gprsconverter.extension.RelationExtension}中EQ或1类型
  String relation;

  /// 与前一个查询条件关系，可空
  /// 后台默认按{@link com.xdja.gprsconverter.extension.SplitJointExtension}中and或1类型
  String splitJoint;
}

/// 查询结果排序方式
///
/// mix与asc、desc不可共存，请避免同时设置
/// asc、desc标签和字段出现的前后顺序会影响排序结果
class Order {
  /// 根据正序排列的字段列表，中间用英文逗号分隔
  String asc;

  /// 根据倒叙排列的字段列表，中间用英文逗号分隔
  String desc;

  /// 混合排序模式，此种模式或忽略asc和desc标签
  String mix;
}
